from tkinter import *
import time

gravity = 3.8
central_thrust_max = 73000.
lateral_thrust_max = 9000.
vmax = 15.

class Game:
    def __init__(self):
        self.fenetre = Tk() #fenetre par defaut. Tk() est un objet
        self.fenetre.title("Simulation de l'atterrissage de Perseverance sur Mars") #titre de la fenetre
        self.fenetre.resizable(False, False) #redimensionnement de la fenetre impossible ni en largeur, ni en hauteur
        self.windowWidth = 700
        self.windowHeight = 700
        self.canevas = Canvas(self.fenetre, width=self.windowWidth, height=self.windowHeight) #parametres de la zone de dessin
        self.canevas.pack() #ajoute la zone de dessin à la fenetre
        self.fond = PhotoImage(file = 'landingSite.gif')
        self.canevas.create_image(0, 0, anchor=NW, image=self.fond)
        self.sprites = []
        self.enfonction = True

    def boucle_principale(self):
        while 1:
            if self.enfonction == True:
                for sprite in self.sprites:
                    sprite.deplacer()
            self.fenetre.update()
            time.sleep(0.05)

class Lander:
    def __init__(self, x, y, vx, vy, image_lander, mass_lander):
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.mass_lander = mass_lander
        self.central_thrust = 0
        self.left_thrust = 0
        self.right_thrust = 0
        self.time = time.time()
        self.widget_lander = jeu.canevas.create_image(self.x, self.y, anchor=NW, image=image_lander)
        self.vtext = jeu.canevas.create_text(10,105, text='vy = '+str(int(self.vy))+' m/s', anchor=NW, font=('Helvetica', '15', 'bold'))

    def deplacer(self):
        if self.y >= landing_site.y1-200: #200 est la hauteur de l'image lander.gif
            if self.vy >= vmax:
                jeu.canevas.create_text(jeu.windowWidth/2,200, text='BOOM !', font=('Helvetica', '48', 'bold'))
            elif self.x < landing_site.x1 or self.x > landing_site.x2-95: #95 est la largeur de l'image lander.gif
                jeu.canevas.create_text(jeu.windowWidth/2,200, text='Zone d\'atterrissage ratee !', font=('Helvetica', '30', 'bold'))
            else:
                jeu.canevas.create_text(jeu.windowWidth/2,200, text='We\'re safe on Mars !', font=('Helvetica', '30', 'bold'))
            jeu.enfonction = False
            return

        now = time.time()
        delta_t = now - self.time
        if central_rocket.allume == True:
            self.central_thrust = central_thrust_max
        if left_rocket.allume == True:
            self.left_thrust = lateral_thrust_max
        if right_rocket.allume == True:
            self.right_thrust = lateral_thrust_max
        if central_rocket.allume == False:
            self.central_thrust = 0
        if left_rocket.allume == False:
            self.left_thrust = 0
        if right_rocket.allume == False:
            self.right_thrust = 0
        delta_vx = delta_t * (self.left_thrust - self.right_thrust)/self.mass_lander
        delta_vy = delta_t * (gravity - self.central_thrust/self.mass_lander)
        self.vx = self.vx + delta_vx
        self.vy = self.vy + delta_vy
        delta_x = delta_t * self.vx
        delta_y = delta_t * self.vy
        self.x = self.x + delta_x
        self.y = self.y + delta_y
        jeu.canevas.move(self.widget_lander, delta_x, delta_y)
        if self.vy > vmax:
            color='red'
        else:
            color='green'
        jeu.canevas.itemconfig(self.vtext, text='vy = '+str(int(self.vy))+' m/s', fill=color)
        self.time = time.time()

class Rectangle:
    def __init__(self, x1, y1, x2, y2, color):
        self.x1 = x1
        self.x2 = x2
        self.y1 = y1
        self.y2 = y2
        jeu.canevas.create_rectangle(self.x1, self.y1, self.x2, self.y2, fill = color)

class Retrorocket:
    #Classe pour creer et deplacer l'image d'une retrofusee
    def __init__(self, image_rocket, x_rocket, y_rocket, key):
        self.x_rocket = x_rocket
        self.y_rocket = y_rocket
        self.image_rocket = image_rocket
        self.rocket=jeu.canevas.create_image(perseverance.x+self.x_rocket, perseverance.y+self.y_rocket, anchor=NW, image=self.image_rocket, state='hidden')
        self.allume = False
        jeu.canevas.bind_all(key, self.allumage) #bind_all : pour lier l'evenement à tous les widgets de l'application

    def allumage(self, evt):
        if self.allume == True:
            self.allume = False
            jeu.canevas.delete(self.rocket)
        else:
            self.allume = True
            if perseverance.y < landing_site.y1-200: #200 est la hauteur de l'image lander.gif
                self.rocket = jeu.canevas.create_image(perseverance.x+self.x_rocket, perseverance.y+self.y_rocket, anchor=NW, image=self.image_rocket)

    def deplacer(self):
        jeu.canevas.delete(self.rocket)
        if self.allume == True  and perseverance.y < landing_site.y1-200: #200 est la hauteur de l'image lander.gif
            self.rocket = jeu.canevas.create_image(perseverance.x+self.x_rocket, perseverance.y+self.y_rocket, anchor=NW, image=self.image_rocket)

class Debris:
    def __init__(self, image_debris):
        self.image_debris = image_debris

    def deplacer(self):
        if perseverance.y >= landing_site.y1-200 and perseverance.vy >= vmax:
            jeu.canevas.delete(perseverance.widget_lander)
            jeu.canevas.create_image(perseverance.x-90, perseverance.y+160, anchor=NW, image=self.image_debris)

jeu = Game()

image_lander = PhotoImage(file = 'lander.gif')
perseverance = Lander(500, 0, 0, 100, image_lander, 1825.)

landing_site = Rectangle(290, 620, 410, 625, 'red')

image_central_rocket = PhotoImage(file = 'pousseeCentrale.gif')
image_left_rocket = PhotoImage(file = 'pousseeLateraleGauche.gif')
image_right_rocket = PhotoImage(file = 'pousseeLateraleDroite.gif')
central_rocket = Retrorocket(image_central_rocket, 22, 48, '<KeyPress-Down>')
left_rocket = Retrorocket(image_left_rocket, -26, 15, '<KeyPress-Left>')
right_rocket = Retrorocket(image_right_rocket, 87, 15, '<KeyPress-Right>')

image_debris = PhotoImage(file = 'debris.gif')
debris = Debris(image_debris)

jeu.sprites.append(perseverance)
jeu.sprites.append(central_rocket)
jeu.sprites.append(left_rocket)
jeu.sprites.append(right_rocket)
jeu.sprites.append(debris)

jeu.boucle_principale()

#jeu.fenetre.mainloop()
