# NAME:
#    	atterrissage_perseverance_objets_en_parametre.py
#
# PURPOSE:
#    	Simulation de l'atterrissage du rover Perseverance dans le cratere Jezero, planete Mars
#       Le but est de faire atterrir le rover sur la barre rouge avec une vitesse verticale vy <= 15.
#
# CALLING SEQUENCE:
#       Dans le Terminal, taper : python3 atterrissage_perseverance_objets_en_parametre.py
#       Ou executer ce programme dans un IDE Python.
#       Pour relancer le jeu, executer de nouveau le programme.
#       Lorsqu'on ferme la fenetre, une erreur apparait, c'est normal.
#
# INPUTS:
#
# OUTPUTS:
#    	Affichage d'une fenetre graphique tkinter
#
# COMMENTS:
#    	La touche Bas active la retrofusee centrale, un nouvel appui sur la touche Bas la desactive.
#       La touche Gauche active la retrofusee gauche, un nouvel appui sur la touche Gauche la desactive.
#       La touche Droite active la retrofusee droite, un nouvel appui sur la touche Droite la desactive.
#       Dans cette version, les objets jeu de la classe Game, perseverance de la classe Lander et landing_site de la classe Rectangle sont passes en parametres dans les autres classes.
#       Pour rendre le jeu plus dynamique, on peut utiliser la combinaison de valeurs : gravity = 38., central_thrust_max = 300000., vmax = 9000.
#
# PROCEDURES AND FUNCTIONS USED:
#
#
# MODIFICATION HISTORY:
#       Programme inspire de https://briggs.net.nz/python-for-kids/puzzles/project1-lander.py.txt
#       Cree entre juin et août 2020 par Thomas Appere thomas.appere@ac-rennes.fr

from tkinter import *
from pygame import *
import time

mixer.init() #pour le son des retrofusees
mixer.music.load("retrofusee.mp3")

gravity = 3.8
central_thrust_max = 73000.
lateral_thrust_max = 9000.
vmax = 15.

class Game:
    def __init__(self):
        self.fenetre = Tk() #fenetre par defaut. Tk() est un objet
        self.fenetre.title("Simulation de l'atterrissage de Perseverance sur Mars") #titre de la fenetre
        self.fenetre.resizable(False, False) #redimensionnement de la fenetre impossible ni en largeur, ni en hauteur
        self.windowWidth = 700
        self.windowHeight = 700
        self.canevas = Canvas(self.fenetre, width=self.windowWidth, height=self.windowHeight) #parametres de la zone de dessin
        self.canevas.pack() #ajoute la zone de dessin à la fenetre
        self.fond = PhotoImage(file = 'landingSite.gif')
        self.canevas.create_image(0, 0, anchor=NW, image=self.fond)
        self.canevas.create_text(self.windowWidth/2,15, text='18 fevrier 2021, cratere Jezero, planete Mars', font=('Helvetica', '20', 'bold'))
        self.canevas.create_text(10,75, text='Pour gagner, il faut atterrir dans la \nzone d\'atterrissage avec vy <= '+str(vmax)+' m/s', anchor=NW)
        self.sprites = []
        self.enfonction = True

    def boucle_principale(self):
        while 1:
            if self.enfonction == True:
                for sprite in self.sprites:
                    sprite.deplacer()
            self.fenetre.update()
            time.sleep(0.05)

class Lander:
    def __init__(self, game, rectangle, x, y, vx, vy, image_lander, mass_lander):
        self.game = game
        self.rectangle = rectangle
        self.x = x
        self.y = y
        self.vx = vx
        self.vy = vy
        self.mass_lander = mass_lander
        self.central_thrust = 0
        self.left_thrust = 0
        self.right_thrust = 0
        self.time = time.time()
        self.image_lander = self.game.canevas.create_image(x, y, anchor=NW, image=image_lander)
        self.vtext = jeu.canevas.create_text(10,105, text='vy = '+str(int(self.vy))+' m/s', anchor=NW, font=('Helvetica', '15', 'bold'))
        self.central_thrust_text = self.game.canevas.create_text(10,35, text='poussee retrofusee centrale = '+str(abs(int(self.central_thrust)))+' N', anchor=NW)
        self.lateral_thrust_text = self.game.canevas.create_text(10,50, text='poussee retrofusees laterales = '+str(int(self.left_thrust) - int(self.right_thrust))+' N', anchor=NW)

    def deplacer(self):
        if self.y >= self.rectangle.y1-200: #200 est la hauteur de l'image lander.gif
            if self.vy >= vmax:
                self.game.canevas.create_text(self.game.windowWidth/2, 200, text='BOOM !', font=('Helvetica', '48', 'bold'))
                mixer.music.stop()
                mixer.music.load("explosion.mp3")
                mixer.music.play()
            elif self.x < self.rectangle.x1 or self.x > self.rectangle.x2-83:
                self.game.canevas.create_text(self.game.windowWidth/2, 200, text='Zone d\'atterrissage ratee !', font=('Helvetica', '30', 'bold'))
                mixer.music.stop()
                mixer.music.load("out_of_landing_site.mp3")
                mixer.music.play()
            else:
                self.game.canevas.create_text(self.game.windowWidth/2, 200, text='We\'re safe on Mars !', font=('Helvetica', '30', 'bold'))
                mixer.music.stop()
                mixer.music.load("touchdown.mp3")
                mixer.music.play()
            self.game.enfonction = False
            return

        now = time.time()
        delta_t = now - self.time
        if central_rocket.allume == True:
            self.central_thrust = central_thrust_max
        if left_rocket.allume == True:
            self.left_thrust = lateral_thrust_max
        if right_rocket.allume == True:
            self.right_thrust = lateral_thrust_max
        if central_rocket.allume == False:
            self.central_thrust = 0
        if left_rocket.allume == False:
            self.left_thrust = 0
        if right_rocket.allume == False:
            self.right_thrust = 0

        if not central_rocket.allume and not left_rocket.allume and not right_rocket.allume: #toutes retrofusees eteintes
            mixer.music.stop()
        else:
            mixer.music.play() #au moins une retrofusee allumee

        delta_vx = delta_t * (self.left_thrust - self.right_thrust)/self.mass_lander
        delta_vy = delta_t * (gravity - self.central_thrust/self.mass_lander)
        self.vx = self.vx + delta_vx
        self.vy = self.vy + delta_vy
        delta_x = delta_t * self.vx
        delta_y = delta_t * self.vy
        self.x = self.x + delta_x
        self.y = self.y + delta_y
        self.game.canevas.move(self.image_lander, delta_x, delta_y)
        if self.vy > vmax:
            color='red'
        else:
            color='green'
        self.game.canevas.itemconfig(self.vtext, text='vy = '+str(int(self.vy))+' m/s', fill=color)
        self.game.canevas.itemconfig(self.central_thrust_text, text ='poussee retrofusee centrale = '+str(abs(int(self.central_thrust)))+' N')
        self.game.canevas.itemconfig(self.lateral_thrust_text, text ='poussee retrofusees laterales = '+str(int(self.left_thrust)-int(self.right_thrust))+' N')
        self.time = time.time()

class Rectangle:
    def __init__(self, game, x1, y1, x2, y2, color):
        self.game = game
        self.x1 = x1
        self.x2 = x2
        self.y1 = y1
        self.y2 = y2
        self.canevas = self.game.canevas
        self.canevas.create_rectangle(self.x1, self.y1, self.x2, self.y2, fill = color)

class Retrorocket:
    #Classe pour creer et deplacer l'image d'une retrofusee
    def __init__(self, game, lander, rectangle, image_rocket, x_rocket, y_rocket, key):
        self.game = game
        self.lander = lander
        self.rectangle = rectangle
        self.x_rocket = x_rocket
        self.y_rocket = y_rocket
        self.image_rocket = image_rocket
        self.rocket=self.game.canevas.create_image(self.lander.x+self.x_rocket, self.lander.y+self.y_rocket, anchor=NW, image=self.image_rocket, state='hidden')
        self.allume = False
        self.game.canevas.bind_all(key, self.allumage) #bind_all : pour lier l'evenement à tous les widgets de l'application

    def allumage(self, evt):
        if self.allume == True:
            self.allume = False
            self.game.canevas.delete(self.rocket)
        else:
            self.allume = True
            if self.lander.y < self.rectangle.y1-200: #200 est la hauteur de l'image lander.gif
                self.rocket = self.game.canevas.create_image(self.lander.x+self.x_rocket, self.lander.y+self.y_rocket, anchor=NW, image=self.image_rocket)

    def deplacer(self):
        self.game.canevas.delete(self.rocket)
        if self.allume == True  and self.lander.y < self.rectangle.y1-200: #200 est la hauteur de l'image lander.gif
            self.rocket = self.game.canevas.create_image(self.lander.x+self.x_rocket, self.lander.y+self.y_rocket, anchor=NW, image=self.image_rocket)

class Debris:
    def __init__(self, game, lander, rectangle, image_debris):
        self.game = game
        self.lander = lander
        self.rectangle = rectangle
        self.image_debris = image_debris

    def deplacer(self):
        if self.lander.y >= self.rectangle.y1-200 and self.lander.vy >= vmax: #200 est la hauteur de l'image lander.gif
            self.game.canevas.delete(self.lander.image_lander)
            self.game.canevas.create_image(self.lander.x-90, self.lander.y+160, anchor=NW, image=self.image_debris)

#Programme principal
jeu = Game()

x1 = 290
y1 = 620
x2 = 410
y2 = 625
landing_site = Rectangle(jeu, x1, y1, x2, y2, 'red')

image_lander = PhotoImage(file = 'lander.gif')
perseverance = Lander(jeu, landing_site, 500, 0, 0, 100, image_lander, 1825.)

image_central_rocket = PhotoImage(file = 'pousseeCentrale.gif')
image_left_rocket = PhotoImage(file = 'pousseeLateraleGauche.gif')
image_right_rocket = PhotoImage(file = 'pousseeLateraleDroite.gif')
central_rocket = Retrorocket(jeu, perseverance, landing_site, image_central_rocket, 22, 48, '<KeyPress-Down>')
left_rocket = Retrorocket(jeu, perseverance, landing_site, image_left_rocket, -26, 15, '<KeyPress-Left>')
right_rocket = Retrorocket(jeu, perseverance, landing_site, image_right_rocket, 87, 15, '<KeyPress-Right>')

image_debris = PhotoImage(file = 'debris.gif')
debris = Debris(jeu, perseverance, landing_site, image_debris)

jeu.sprites.append(perseverance)
jeu.sprites.append(central_rocket)
jeu.sprites.append(left_rocket)
jeu.sprites.append(right_rocket)
jeu.sprites.append(debris)

jeu.boucle_principale()

#jeu.fenetre.mainloop()
